package gregapi.item.prefixitem;

import static gregapi.data.CS.*;
import gregapi.code.ItemStackContainer;
import gregapi.data.LH;
import gregapi.data.MT;
import gregapi.data.TD;
import gregapi.item.CreativeTab;
import gregapi.item.IItemGT;
import gregapi.item.IItemNoGTOverride;
import gregapi.item.IItemUpdatable;
import gregapi.lang.LanguageHandler;
import gregapi.oredict.*;
import gregapi.util.OM;
import gregapi.util.UT;

import java.util.List;

import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.IIcon;
import net.minecraft.world.World;
import cpw.mods.fml.common.registry.GameRegistry;

/**
 * @author Gregorius Techneticies
 */
public class PrefixItem extends Item implements Runnable, IItemUpdatable, IItemGT, IItemNoGTOverride, IOreDictItemDataOverrideItem, IOreDictOptimizedParsingItem {
	public final String mNameInternal;
	public final OreDictPrefix mPrefix;
	
	public ItemStack mContainerItem = null;
	
	/** The Sound played when crafting with this Item */
	public String mCraftingSound = null;
	
	/**
	 * 
	 * @param aModIDOwner the ID of the owning Mod. DO NOT INSERT ANY GREGTECH MODID!!!
	 * @param aModIDTextures the ID of the Texture providing Mod (for the "ModID:TextureName" thing)
	 * @param aNameInternal the internal Name of this Item. DO NOT START YOUR UNLOCALISED NAME WITH "gt."!!!
	 * @param aPrefix the OreDictPrefix corresponding to this Item.
	 */
	public PrefixItem(String aModIDOwner, String aModIDTextures, String aNameInternal, OreDictPrefix aPrefix) {
		super();
		mPrefix = aPrefix;
		mNameInternal = aNameInternal;
		GameRegistry.registerItem(this, mNameInternal, aModIDOwner);
		
		setMaxDamage(0);
        setHasSubtypes(T);
		mPrefix.addTextureSet(aModIDTextures, T);
		LH.add("oredict." + mPrefix.dat(MT.Empty).toString() + ".name", getLocalName(mPrefix, MT.Empty));
		LH.add(mNameInternal+"."+W+".name", "Any Sub-Item of this one"); // Local Name for the WildcardItem Variant.
        mPrefix.mRegisteredItems.add(new ItemStackContainer(this, 1, W)); // this optimises some processes by decreasing the size of the Set.
        
        if (mPrefix.mCreativeTab == null) mPrefix.mCreativeTab = new CreativeTab(mPrefix.mNameInternal, mPrefix.mNameCategory, this, W);
		setCreativeTab(mPrefix.mCreativeTab);
		
		// Execute before all the other things. This is to ensure that PrefixItems are created before MultiItems.
		GAPI.mBeforeInit.add(0, this);
	}
	
	/** This ensures, that all Materials are registered at the time this Item registers to the OreDictionary. */
	@Override
	public void run() {
		boolean tUnificationAllowed = (mPrefix.contains(TD.Prefix.UNIFICATABLE) && !mPrefix.contains(TD.Prefix.UNIFICATABLE_RECIPES));
		for (short i = 0; i < OreDictMaterial.MATERIAL_ARRAY.length; i++) if (OreDictMaterial.MATERIAL_ARRAY[i] != null && mPrefix.isGeneratingItem(OreDictMaterial.MATERIAL_ARRAY[i])) {
			ItemStack tStack = UT.Stacks.make(this, 1, i);
			UT.Stacks.update_(tStack);
			LH.add("oredict." + mPrefix.dat(OreDictMaterial.MATERIAL_ARRAY[i]).toString() + ".name", getLocalName(mPrefix, OreDictMaterial.MATERIAL_ARRAY[i]));
			if (tUnificationAllowed) OreDictManager.INSTANCE.addTarget_(mPrefix, OreDictMaterial.MATERIAL_ARRAY[i], tStack); else OreDictManager.INSTANCE.registerOre_(mPrefix, OreDictMaterial.MATERIAL_ARRAY[i], tStack);
		}
	}
	
	@Override
    public void getSubItems(Item var1, CreativeTabs aCreativeTab, List aList) {
        if ((SHOW_HIDDEN_PREFIXES || !mPrefix.contains(TD.Creative.HIDDEN)) && (SHOW_ORE_BLOCK_PREFIXES || !mPrefix.contains(TD.Prefix.ORE))) for (int i = 0; i < OreDictMaterial.MATERIAL_ARRAY.length; i++) if (mPrefix.isGeneratingItem(OreDictMaterial.MATERIAL_ARRAY[i])) if (SHOW_HIDDEN_MATERIALS || !OreDictMaterial.MATERIAL_ARRAY[i].contains(TD.Creative.HIDDEN)) {
			ItemStack tStack = OM.get_(UT.Stacks.make(this, 1, i));
			if (tStack.getItem() == this) {
				updateItemStack(tStack);
	        	aList.add(tStack);
			}
        }
        if (aList.isEmpty()) UT.Stacks.hide(this);
    }
	
	@Override
    public int getRenderPasses(int metadata) {
        return 2;
    }
    
	@Override
    public boolean requiresMultipleRenderPasses() {
        return mPrefix.mIconIndexItem >= 0;
    }
	
	@Override
    public void registerIcons(IIconRegister aIconRegister) {/*
		if (mPrefix.mIconIndexItem >= 0) {
			MT.NULL.mTextureSetsItems.get(mPrefix.mIconIndexItem).registerIcons(aIconRegister);
			HashSet<IIconContainer> tSet = new HashSet<IIconContainer>();
			for (int i = 0; i < OreDictMaterial.MATERIAL_ARRAY.length; i++) if (OreDictMaterial.MATERIAL_ARRAY[i] != null && OreDictMaterial.MATERIAL_ARRAY[i].mTextureSetsItems != null) {
				IIconContainer tIcon = OreDictMaterial.MATERIAL_ARRAY[i].mTextureSetsItems.get(mPrefix.mIconIndexItem);
				if (!tSet.contains(tIcon)) {tIcon.registerIcons(aIconRegister); tSet.add(tIcon);}
			}
		}*/
    }
	
	@Override
    public int getSpriteNumber() {
        return 1;
    }
	
	@Override
    public IIcon getIconIndex(ItemStack aStack) {
        return getIconFromDamageForRenderPass(UT.Stacks.meta(aStack), 0);
    }
	
	@Override
    public IIcon getIconFromDamage(int aMetaData) {
    	return getIconFromDamageForRenderPass(aMetaData, 0);
    }
	
	@Override
    public IIcon getIcon(ItemStack aStack, int aRenderPass) {
        return getIconFromDamageForRenderPass(UT.Stacks.meta(aStack), aRenderPass);
    }
	
	@Override
    public IIcon getIcon(ItemStack aStack, int aRenderPass, EntityPlayer aPlayer, ItemStack aUsedStack, int aUseRemaining) {
        return getIconFromDamageForRenderPass(UT.Stacks.meta(aStack), aRenderPass);
    }
	
    @Override
    public IIcon getIconFromDamageForRenderPass(int aMetaData, int aRenderPass) {
		if (mPrefix.mIconIndexItem >= 0) {
			if (UT.Code.exists(aMetaData, OreDictMaterial.MATERIAL_ARRAY) && OreDictMaterial.MATERIAL_ARRAY[aMetaData].mTextureSetsItems != null)
			return OreDictMaterial.MATERIAL_ARRAY[aMetaData]	.mTextureSetsItems.get(mPrefix.mIconIndexItem).getIcon(aRenderPass);
			return MT.NULL										.mTextureSetsItems.get(mPrefix.mIconIndexItem).getIcon(aRenderPass);
		}
        return null;
    }
	
	@Override
    public int getColorFromItemStack(ItemStack aStack, int aRenderPass) {
		if (aRenderPass == 0) {
			short aMetaData = UT.Stacks.meta(aStack);
			if (UT.Code.exists(aMetaData, OreDictMaterial.MATERIAL_ARRAY)) return UT.Code.getRGBInt(OreDictMaterial.MATERIAL_ARRAY[aMetaData].mRGBa[mPrefix.mState]);
		}
		return 16777215;
    }
	
	@Override
	public final String getUnlocalizedName(ItemStack aStack) {
		short aMetaData = UT.Stacks.meta(aStack);
		if (aMetaData == W) return mNameInternal+"."+W;
		if (UT.Code.exists(aMetaData, OreDictMaterial.MATERIAL_ARRAY)) return "oredict." + mPrefix.dat(OreDictMaterial.MATERIAL_ARRAY[aMetaData]).toString();
		return mNameInternal;
	}
	
	@Override
    public ItemStack getContainerItem(ItemStack aStack) {
		if (UT.Stacks.equal(aStack, mContainerItem, T)) return null;
		if (mCraftingSound != null) UT.Sounds.play(mCraftingSound, 20, 1.0F);
		return mContainerItem != null ? UT.Stacks.amount(1, mContainerItem) : mPrefix.mContainerItem != null ? UT.Stacks.amount(1, mContainerItem = mPrefix.mContainerItem) : null;
    }
	
	@Override
	public OreDictItemData getOreDictItemData(ItemStack aStack) {
		if (UT.Code.exists(UT.Stacks.meta(aStack), OreDictMaterial.MATERIAL_ARRAY)) return new OreDictItemData(mPrefix, OreDictMaterial.MATERIAL_ARRAY[UT.Stacks.meta(aStack)]);
		return null;
	}
	
	@Override public String toString() {return mNameInternal;}
	@Override public final String getUnlocalizedName() {return mNameInternal;}
	@Override public final Item setUnlocalizedName(String aName) {return this;}
	@Override public final boolean hasContainerItem(ItemStack aStack) {return getContainerItem(aStack) != null;}
	@Override public void updateItemStack(ItemStack aStack) {int aDamage = UT.Stacks.meta(aStack); if (UT.Code.exists(aDamage, OreDictMaterial.MATERIAL_ARRAY)) {OreDictMaterial aMaterial = OreDictMaterial.MATERIAL_ARRAY[aDamage]; if (aMaterial != aMaterial.mTargetRegistration) aStack.setItemDamage(aMaterial.mTargetRegistration.mID);}}
	@Override public void onCreated(ItemStack aStack, World aWorld, EntityPlayer aPlayer) {updateItemStack(aStack);}
	@Override public void addInformation(ItemStack aStack, EntityPlayer aPlayer, List aList, boolean aF3_H) {/**/}
	@Override public boolean isBookEnchantable(ItemStack aStack, ItemStack aBook) {return false;}
	@Override public boolean getIsRepairable(ItemStack aStack, ItemStack aMaterial) {return false;}
	@Override public int getItemEnchantability() {return 0;}
	@Override public int getItemStackLimit(ItemStack aStack) {return mPrefix.mDefaultStackSize;}
	@Override public OreDictMaterial getMaterial(int aMetaData) {return UT.Code.exists(aMetaData, OreDictMaterial.MATERIAL_ARRAY) ? OreDictMaterial.MATERIAL_ARRAY[aMetaData] : null;}
	@Override public OreDictPrefix getPrefix(int aMetaData) {return mPrefix;}
	/*
	@Override @Optional.Method(modid = ModIDs.TC) public void setAspects(ItemStack aStack, AspectList aAspectList) {}
	
	@Override @Optional.Method(modid = ModIDs.TC)
	public AspectList getAspects(ItemStack aStack) {
	    List<TC_AspectStack> rAspects = new ArrayListNoNulls<TC_AspectStack>();
	    for (TC_AspectStack tAspect : mPrefix.mAspects) tAspect.addToAspectList(rAspects);
	    OreDictMaterial aMaterial = (UT.Code.exists(aStack), OreDictMaterial.MATERIAL_ARRAY) ? OreDictMaterial.MATERIAL_ARRAY[UT.Stacks.meta(aStack)] : null);
	    if (aMaterial != null && (mPrefix.mAmount >= U || mPrefix.mAmount < 0)) for (TC_AspectStack tAspect : aMaterial.mAspects) tAspect.addToAspectList(rAspects);
		
	    for (TC_AspectStack tAspect : rAspects) tAspect.mAmount = Math.min(10, tAspect.mAmount);
	    return (AspectList)GT_API.sCompatTC.getAspectList(rAspects);
	}
	*/
	
	/** @return the Local Name for this Item depending on Prefix and Material. */
	public String getLocalName(OreDictPrefix aPrefix, OreDictMaterial aMaterial) {
		return LanguageHandler.getLocalName(aPrefix, aMaterial);
	}
}